﻿using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Sharing.Share.Dtos.WebPortal;
using Sharing.WebCore;
using Sharing.WebPortal.Dtos;
using SqlSugar;
using XF.Domain.Repository;

namespace Sharing.WebPortal.Controllers
{
    /// <summary>
    /// 通知公告
    /// </summary>
    public class ArticleController : BaseController
    {
        private readonly IMapper _mapper;
        private readonly IRepository<Bulletin> _bulletinRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="bulletinRepository"></param>
        public ArticleController(IMapper mapper, IRepository<Bulletin> bulletinRepository)
        {
            _mapper = mapper;
            _bulletinRepository = bulletinRepository;
        }

        /// <summary>
        /// 获取列表
        /// </summary>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getarticlelist")]
        public async Task<DeReponse<ArticleListDataDto>> GetArticleList([FromBody] QueryArticleListDto dto)
        {
            //  Kv kv = new() { Key = "2", Value = "门户网站" };
            RefAsync<int> total = 0;
            var items = await _bulletinRepository.Queryable()
                .Where(p => p.LoseEfficacyTime >= DateTime.Now)
                 .Where(p => p.PushRanges.Contains(new() { Key = "2", Value = "门户网站" }))
                 .Where(p => p.BulletinTypeId == dto.NoticeType)
                 .WhereIF(!string.IsNullOrEmpty(dto.Condition), p => p.Title.Contains(dto.Condition))
                 .OrderByDescending(p => p.BulletinTime)
                 .Select(it => new
                 {
                     Page = SqlFunc.MappingColumn(default(int), "  row_number()  over( order by 'NoticeCreateDate' desc ) "),
                     NoticeID = it.Id,
                     NoticeContent = it.Content,
                     NoticeTypeName = it.BulletinTypeName,
                     NoticeTitle = it.Title,
                     NoticeBMName = it.SourceOrgName,
                     NoticeCreateDate = it.BulletinTime,
                     VideoUrl = "",
                     WNLT_FullCode = ""
                 })
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total);

            //计算总页数
            int nPageCount = Convert.ToInt32(Math.Ceiling(Convert.ToDouble(total) / dto.PageSize));
            ArticleListDataDto dataDto = new()
            {
                PageCount = nPageCount,
                data = _mapper.Map<IReadOnlyList<ArticleListDto>>(items)
            };

            return DeReponse<ArticleListDataDto>.Success(dataDto);
        }

        /// <summary>
        /// 获取详情，修改阅读次数
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getarticledetails")]
        public async Task<DeReponse<ArticleDetailsDto>> GetArticleDetails(string id)
        {
            var data = await _bulletinRepository.GetAsync(p => p.Id == id, HttpContext.RequestAborted);

            if (data != null)
            {
                data.ReadedNum = data.ReadedNum++;
                await _bulletinRepository.UpdateAsync(data, HttpContext.RequestAborted);
            }

            ArticleDetailsDto detailsDto = new()
            {
                NoticeID = data.Id,
                NoticeTypeName = data.BulletinTypeName,
                NoticeTitle = data.Title,
                NoticeBMName = data.SourceOrgName,
                NoticeCreateDate = data.BulletinTime,
                NoticeRCount = data.ReadedNum,
                WNED_VideoUrl = "",
                NoticeContent = data.Content
            };

            return DeReponse<ArticleDetailsDto>.Success(detailsDto);
        }

        /// <summary>
        /// 上一条和下一条
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getpreviousandnext")]
        public async Task<DeReponse<List<ArticlePreviousAndNextDataDto>>> GetPreviousAndNext([FromBody] ArticlePreviousAndNextDto dto)
        {
            var sugar = _bulletinRepository.Queryable()
                .Where(p => p.LoseEfficacyTime >= DateTime.Now)
                 .Where(p => p.PushRanges.Contains(new() { Key = "2", Value = "门户网站" }));
            if (dto.FullSearch == "1")//全文搜索
            {
                sugar.Where(p => p.BulletinTypeId == "1" || p.BulletinTypeId == "5" || p.BulletinTypeId == "6" || p.BulletinTypeId == "7" || p.BulletinTypeId == "3")
                    .WhereIF(!string.IsNullOrEmpty(dto.Condition), p => p.Content.Contains(dto.Condition) || p.Title.Contains(dto.Condition));
            }
            else//指定分类
            {
                sugar.WhereIF(!string.IsNullOrEmpty(dto.Condition), p => p.Title.Contains(dto.Condition))
                    .Where(p => p.BulletinTypeId == dto.NoticeTypeId);
            }

            var list = await sugar.OrderByDescending(p => p.BulletinTime)
                .Select(it => new
                {
                    Page = SqlFunc.MappingColumn(default(int), "  row_number()  over( order by 'NoticeCreateDate' desc ) "),
                    NoticeID = it.Id,
                    NoticeTitle = it.Title,
                    NoticeCreateDate = it.BulletinTime
                })
                .ToListAsync();
            //数据为空返回空数据
            if (list == null || list.Count == 0)
                return DeReponse<List<ArticlePreviousAndNextDataDto>>.Success(null);
            else
            {
                var temp = list.Find(p => p.NoticeID == dto.ID);
                if (temp != null)
                {
                    List<ArticlePreviousAndNextDataDto> returnDto = new();
                    //上一条
                    var pTemp = list.Find(p => p.Page == temp.Page - 1);
                    if (pTemp != null)

                        returnDto.Add(new()
                        {
                            NoticeID = pTemp.NoticeID,
                            NoticeTitle = pTemp.NoticeTitle,
                            pntype = "p"
                        });

                    //下一条
                    var nTemp = list.Find(p => p.Page == temp.Page + 1);
                    if (nTemp != null)
                        returnDto.Add(new()
                        {
                            NoticeID = nTemp.NoticeID,
                            NoticeTitle = nTemp.NoticeTitle,
                            pntype = "n"
                        });
                    return DeReponse<List<ArticlePreviousAndNextDataDto>>.Success(returnDto);
                }
                else
                    return DeReponse<List<ArticlePreviousAndNextDataDto>>.Success(null);
            }
        }

        /// <summary>
        /// 全文检索
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getfulltextsearchlist")]
        public async Task<DeReponse<FullTextSearchListDataDto>> GetFullTextSearchList([FromBody] QueryArticleListDto dto)
        {
            if (string.IsNullOrEmpty(dto.Condition))
                return DeReponse<FullTextSearchListDataDto>.Success(null);

            RefAsync<int> total = 0;
            var items = await _bulletinRepository.Queryable()
                .Where(p => p.LoseEfficacyTime >= DateTime.Now)
                 .Where(p => p.PushRanges.Contains(new() { Key = "2", Value = "门户网站" }))
                 .Where(p => p.BulletinTypeId == "1" || p.BulletinTypeId == "5" || p.BulletinTypeId == "6" || p.BulletinTypeId == "7" || p.BulletinTypeId == "3")
                 .WhereIF(!string.IsNullOrEmpty(dto.Condition), p => p.Title.Contains(dto.Condition))
                 .OrderByDescending(p => p.BulletinTime)
                 .Select(it => new
                 {
                     Page = SqlFunc.MappingColumn(default(int), "  row_number()  over( order by 'NoticeCreateDate' desc ) "),
                     NoticeID = it.Id,
                     Content = it.Content,
                     NoticeTypeID = it.BulletinTypeId,
                     NoticeTypeName = it.BulletinTypeName,
                     NoticeTitle = it.Title,
                     NoticeBMName = it.SourceOrgName,
                     NoticeCreateDate = it.BulletinTime,
                     TypeCode = "",
                 })
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total);

            //计算总页数
            int nPageCount = Convert.ToInt32(Math.Ceiling(Convert.ToDouble(total) / dto.PageSize));
            FullTextSearchListDataDto dataDto = new()
            {
                PageCount = nPageCount,
                data = _mapper.Map<IReadOnlyList<FullTextSearchListDto>>(items)
            };
            return DeReponse<FullTextSearchListDataDto>.Success(dataDto);
        }
    }
}
